@@ -69,6 +69,7 @@ class ScrapyPlaywrightDownloadHandler(HTTPDownloadHandler):
         self.stats = crawler.stats
 
         self.browser_type: str = crawler.settings.get("PLAYWRIGHT_BROWSER_TYPE") or "chromium"
+        self.page_load_until: str = crawler.settings.get("PLAYWRIGHT_PAGE_LOAD_UNTIL") or "load"
         self.max_pages_per_context: int = crawler.settings.getint(
             "PLAYWRIGHT_MAX_PAGES_PER_CONTEXT"
         ) or crawler.settings.getint("CONCURRENT_REQUESTS")
@@ -153,7 +154,8 @@ class ScrapyPlaywrightDownloadHandler(HTTPDownloadHandler):
             if self.user_agent_rotation_idx >= len(self.playwright_user_agents):
                 self.user_agent_rotation_idx = 0
 
-        self.context_semaphores['default'].release()
+        if self.context_semaphores['default'].locked():
+            self.context_semaphores['default'].release()
         logger.info("Re create browser: Completed")
 
     async def _create_page(self, request: Request) -> Page:
@@ -241,23 +243,20 @@ class ScrapyPlaywrightDownloadHandler(HTTPDownloadHandler):
             logger.exception(f"[{request.url}] Error while downloading request")
             return Response('', status=504)
         else:
-            logger.info(f"[{request.url}] Return result")
-            return result
-        finally:
-            logger.info(f"[{request.url}] Finally start")
             try:
                 if page is not None:
                     if not page.is_closed():
                         await page.close()
             except:
-                logging.exception("Error on finally")
-            finally:
+                logger.error(f"[{request.url}] Error on page close")
+            return result
+        finally:
+            if self.context_semaphores['default'].locked():
                 self.context_semaphores['default'].release()
-            logger.info(f"[{request.url}] Finally end")
 
     async def _download_request_with_page(self, request: Request, page: Page) -> Response:
         start_time = time()
-        response = await page.goto(request.url, wait_until="domcontentloaded")
+        response = await page.goto(request.url, wait_until=self.page_load_until)
 
         logger.info(f"[{request.url}] After goto")
         await self._apply_page_methods(page, request)
@@ -334,7 +333,7 @@ class ScrapyPlaywrightDownloadHandler(HTTPDownloadHandler):
     def _make_close_page_callback(self, context_name: str) -> Callable:
         def close_page_callback() -> None:
             logger.info("Close page callback")
-            if context_name in self.context_semaphores:
+            if context_name in self.context_semaphores and self.context_semaphores[context_name].locked():
                 self.context_semaphores[context_name].release()
 
         return close_page_callback
